"""Tests for the `physbiblio.webimport` package.

This file is part of the physbiblio package.
"""
from physbiblio import __author__, __email__, __version__, __version_date__

__all__ = ["test_webimport"]
