"""The package that contains all the GUI modules used by the PhysBiblio.

This file is part of the physbiblio package.
"""
from physbiblio import __author__, __email__, __version__, __version_date__

__all__ = [
    "basicDialogs",
    "bibWindows",
    "catWindows",
    "commonClasses",
    "dialogWindows",
    "errorManager",
    "expWindows",
    "inspireStatsGUI",
    "mainWindow",
    "marks",
    "profilesManager",
    "resourcesPyside2",
    "setuptests",
    "threadElements",
]
